@@ -12,8 +12,12 @@ from typing import Any, Dict, List, Optional, Union
 import dateutil
 import numpy as np
 import pandas as pd
+import pyarrow
+import pyarrow.feather
 from ecl.summary import EclSum, EclSumKeyWordVector
 
+from ecl2df import getLogger_ecl2csv
+
 from . import parameters
 from .common import write_dframe_stdout_file
 from .eclfiles import EclFiles
@@ -76,6 +80,7 @@ def _ensure_date_or_none(some_date: Optional[Union[str, dt.date]]) -> Optional[d
         return None
     if isinstance(some_date, str):
         return dateutil.parser.parse(some_date).date()  # type: ignore
+    raise TypeError(f"some_date must be a string or a date, got {some_date}")
 
 
 def _crop_datelist(
@@ -419,9 +424,62 @@ def df(
     if datetime is True:
         if dframe.index.dtype == "object":
             dframe.index = pd.to_datetime(dframe.index)
+
+    # Remove duplicated column names. These will occur from libecl
+    # when the user has repeated vector names in the summary SECTION
+    dupes = dframe.columns.duplicated()
+    if dupes.any():
+        logger.warning(
+            "Duplicated columns detected, check your DATA file "
+            "for repeated vectors in the SUMMARY section"
+        )
+        logger.warning("Duplicates: %s", list(dframe.columns[dupes]))
+        dframe = dframe.loc[:, ~dframe.columns.duplicated()]
     return dframe
 
 
+def _df2pyarrow(dframe: pd.DataFrame) -> pyarrow.Table:
+    """Construct a Pyarrow table from a dataframe, conserving metadata.
+
+    All integer columns will have datatype int32, all floats will have float32
+    as this is Eclipse specific.
+
+    Metadata values will be written as strings. A True property is thus
+    represented as the string "True" in the arrow object.
+
+    The index in the dataframe is always assumed to be a time-index, but
+    not necessarily a Pandas datetimetype (which is only of nanosecond precision).
+    This index is always named DATE in the pyarrow table.
+    """
+
+    field_list: List[pyarrow.Field] = []
+    field_list.append(pyarrow.field("DATE", pyarrow.timestamp("ms")))
+    column_arrays = [dframe.index.to_numpy()]
+
+    for colname in dframe.columns:
+        if "meta" in dframe.attrs and colname in dframe.attrs["meta"]:
+            # Boolean objects in the metadata dictionary must be converted to bytes:
+            field_metadata = {
+                bytes(key, encoding="ascii"): bytes(str(value), encoding="ascii")
+                for key, value in dframe.attrs["meta"][colname].items()
+            }
+        else:
+            field_metadata = {}
+        if pd.api.types.is_integer_dtype(dframe.dtypes[colname]):
+            dtype = pyarrow.int32()
+        elif pd.api.types.is_string_dtype(dframe.dtypes[colname]):
+            # Parameters are potentially merged into the dataframe.
+            dtype = pyarrow.string()
+        else:
+            dtype = pyarrow.float32()
+        field_list.append(pyarrow.field(colname, dtype, metadata=field_metadata))
+        column_arrays.append(dframe[colname].to_numpy())
+
+    schema = pyarrow.schema(field_list)
+
+    return pyarrow.table(column_arrays, schema=schema)
+
+
 def _merge_params(
     dframe: pd.DataFrame,
     paramfile: Optional[Union[str, Path]] = None,
@@ -446,7 +504,7 @@ def _merge_params(
         logger.info("Loading parameters from files: %s", str(param_files))
         param_dict = parameters.load_all(param_files)
     elif paramfile is not None and Path(paramfile).is_absolute():
-        logger.info("Loading parameter from file: %s", str(paramfile))
+        logger.info("Loading parameters from file: %s", str(paramfile))
         param_dict = parameters.load(paramfile)
     else:
         raise ValueError("Not able to locate parameters.txt")
@@ -540,8 +598,8 @@ def _fix_dframe_for_libecl(dframe: pd.DataFrame) -> pd.DataFrame:
         dframe.index.values[0], (dt.datetime, np.datetime64, pd.Timestamp)
     ):
         raise ValueError(
-            "dataframe must have a datetime index, got %s of type %s"
-            % (dframe.index.values[0], type(dframe.index.values[0]))
+            "dataframe must have a datetime index, got "
+            f"{dframe.index.values[0]} of type {type(dframe.index.values[0])}"
         )
     dframe.sort_index(axis=0, inplace=True)
 
@@ -607,9 +665,6 @@ def _libecl_eclsum_pandas_frame(
         for key in column_keys:
             keywords.add_keywords(key)
 
-    if len(keywords) == 0:
-        raise ValueError("No valid key")
-
     # pylint: disable=protected-access
     if time_index is None:
         time_index = eclsum.dates  # Changed from libecl
@@ -726,7 +781,7 @@ def fill_parser(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
         "--end_date",
         type=str,
         help=(
-            "End at a specific date, in ISO format YYYY-MM-DD"
+            "End at a specific date, in ISO format YYYY-MM-DD. "
             "Ignored if time_index is first or last"
         ),
         default="",
@@ -735,14 +790,14 @@ def fill_parser(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
         "-p",
         "--params",
         action="store_true",
-        help="Merge key-value data from parameter file into each row",
+        help="Merge key-value data from parameter file into each row.",
     )
     parser.add_argument(
         "--paramfile",
         type=str,
         help=(
             "Filename of key-value parameter file to look for if -p is set, "
-            "relative to Eclipse DATA file or an absolute filename "
+            "relative to Eclipse DATA file or an absolute filename. "
             "If not supplied, parameters.{json,yml,txt} in "
             "{., .. and ../..} will be merged in."
         ),
@@ -753,11 +808,12 @@ def fill_parser(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
         "--output",
         type=str,
         help=(
-            "Name of output csv file. Use '-' to write to stdout. "
-            "Default 'summary.csv'"
+            "Name of output file. Use '-' to write to stdout. " "Default 'summary.csv'"
         ),
         default="summary.csv",
     )
+    parser.add_argument("--arrow", action="store_true", help="Write to pyarrow format")
+
     parser.add_argument("-v", "--verbose", action="store_true", help="Be verbose")
     return parser
 
@@ -780,11 +836,13 @@ def fill_reverse_parser(parser: argparse.ArgumentParser) -> argparse.ArgumentPar
 
 def summary_main(args) -> None:
     """Read summary data from disk and write CSV back to disk"""
-    if args.verbose:
-        logging.basicConfig(level=logging.INFO)
+    logger = getLogger_ecl2csv(  # pylint: disable=redefined-outer-name
+        __name__, vars(args)
+    )
     eclbase = (
         args.DATAFILE.replace(".DATA", "").replace(".UNSMRY", "").replace(".SMSPEC", "")
     )
+
     eclfiles = EclFiles(eclbase)
     sum_df = df(
         eclfiles,
@@ -796,15 +854,17 @@ def summary_main(args) -> None:
         paramfile=args.paramfile,
         datetime=False,
     )
+    if args.arrow:
+        sum_df = _df2pyarrow(sum_df)
+
     write_dframe_stdout_file(sum_df, args.output, index=True, caller_logger=logger)
 
 
 def summary_reverse_main(args) -> None:
     """Entry point for usage with "csv2ecl summary" on the command line"""
-    if args.verbose and not args.debug:
-        logging.basicConfig(level=logging.INFO)
-    if args.debug:
-        logging.basicConfig(level=logging.DEBUG)
+    logger = getLogger_ecl2csv(  # pylint: disable=redefined-outer-name
+        __name__, vars(args)
+    )
 
     summary_df = pd.read_csv(args.csvfile)
     logger.info("Parsed %s", args.csvfile)
